#include "buddy.h"


#include "param.h"
#include "sleeplock.h"
#include "riscv.h"
#include "defs.h"
#include "fs.h"
#include "buf.h"

struct buddy buddy_system;

void buddy_init(uint64 base){
    if((base & 0xfff) != 0){
        panic("base not page aligned");
    }
    initlock(&buddy_system.lock, "buddy");
    buddy_system.base_addr = base;
    int j = 0;
    for(int i = 0; i < BUDDY_SYSTEM_SIZE; i++){
        for(int k = 0; k < (1 << i); k++){
            buddy_system.longest[j+k] = 1 << (BUDDY_SYSTEM_SIZE - i);
        }
        j += (1 << i);
    }
}


// size: number of pages
uint64 buddy_alloc(uint64 size){
    acquire(&buddy_system.lock);
    uint32 i = 0;
    uint32 offset = 0;
    uint32 layer = BUDDY_SYSTEM_SIZE - 1;
    if(buddy_system.longest[i] < size){
        release(&buddy_system.lock);
        return 0;
    }
    while(buddy_system.longest[i] >= size && layer > 0){
        if(buddy_system.longest[LEFT(i)] >= size){
            i = LEFT(i);
        } else {
            i = RIGHT(i);
            offset += (1 << layer);
        } 
        layer--;
    }
    buddy_system.longest[i] = 0;
    while(i != 0){
        uint32 parent = PARENT(i);
        uint32 left = buddy_system.longest[LEFT(parent)];
        uint32 right = buddy_system.longest[RIGHT(parent)];
        buddy_system.longest[parent] = MAX(left, right);
        i = parent;
    }
    release(&buddy_system.lock);
    return (uint64)((char*) buddy_system.base_addr + (offset << 12));
}

void buddy_free(uint64 addr){
    acquire(&buddy_system.lock);
    uint32 offset = (addr - buddy_system.base_addr) >> 12;
    uint32 idx = offset + (1 << BUDDY_SYSTEM_SIZE) - 1;
    uint32 count = 1;
    while(buddy_system.longest[idx] != 0){
        if(idx == 0){
            release(&buddy_system.lock);
            return;
        }
        idx = PARENT(idx);
        count <<= 1;
    }
    buddy_system.longest[idx] = count; 
    while(idx != 0){
        uint32 parent = PARENT(idx);
        uint32 left = buddy_system.longest[LEFT(parent)];
        uint32 right = buddy_system.longest[RIGHT(parent)];
        if(left == count && right == count)
            buddy_system.longest[parent] = count << 1;
        else
            buddy_system.longest[parent] = count; 
        idx = parent;
        count <<= 1;
    }
    release(&buddy_system.lock);
}
