// Physical memory allocator, for user processes,
// kernel stacks, page-table pages,
// and pipe buffers. Allocates whole 4096-byte pages.

#include "types.h"
#include "param.h"
#include "memlayout.h"
#include "include/spinlock.h"
#include "riscv.h"
#include "defs.h"
#include "buddy2.h"
#include "slub.h"



void freerange(void *pa_start, void *pa_end);

extern char end[]; // first address after kernel.
                   // defined by kernel.ld.

struct run {
  struct run *next;
};

struct {
  struct spinlock lock;
  struct run *freelist;
} kmem;

void
kinit()
{
  buddy_init_((uint64)end);
  initlock(&kmem.lock, "kmem");
  slub_allocator_init();
  initlock(&pages_lock, "pages");
  // freerange(end, (void*)PHYSTOP);
}

// void
// freerange(void *pa_start, void *pa_end)
// {
//   char *p;
//   p = (char*)PGROUNDUP((uint64)pa_start);
//   for(; p + PGSIZE <= (char*)pa_end; p += PGSIZE)
//     kfree(p);
// }

// Free the page of physical memory pointed at by pa,
// which normally should have been returned by a
// call to kalloc().  (The exception is when
// initializing the allocator; see kinit above.)
void
kfree(void *pa)
{
  // struct run *r;

  if((char*)pa < end || (uint64)pa >= PHYSTOP)
    panic("kfree");
  acquire(&pages_lock);
  if(pages[PA2PGREF_ID((uint64)pa)].refcount <= 1){
    // memset(pa, 1, );
    // r = (struct run*)pa;
    // acquire(&kmem.lock);
    // r->next = kmem.freelist;
    // kmem.freelist = r;
    // buddy_free_((uint64)pa);
    kfree_(pa);//清零的工作交给kfree_
    // release(&kmem.lock);
  }
  if(pages[PA2PGREF_ID((uint64)pa)].refcount >= 1)
    pages[PA2PGREF_ID((uint64)pa)].refcount--;
  release(&pages_lock);
}

// Allocate one 4096-byte page of physical memory.
// Returns a pointer that the kernel can use.
// Returns 0 if the memory cannot be allocated.
void *
kalloc(void)
{
  // struct run *r;
  uint64 pa;
  acquire(&pages_lock);
  pa = (uint64)kmalloc_(PGSIZE);
  // r = kmem.freelist;
  // if(r)
  //   kmem.freelist = r->next;
  if(pa)
    memset((char*)pa, 5, PGSIZE); // fill with junk
  pages[PA2PGREF_ID(pa)].refcount = 1;
  release(&pages_lock);
  // printf("%p, end:%p \n", (void*)pa, (void*)end);
  return (void*)pa;
}

//只允许使用这个函数创建用于内核内部的数据结构，不允许用它给用户分配空间。
void *
kmalloc(uint32 size)
{
  // struct run *r;
  uint64 pa;
  acquire(&pages_lock);
  pa = (uint64)kmalloc_(size);
  // r = kmem.freelist;
  // if(r)
  //   kmem.freelist = r->next;
  if(pa)
    memset((char*)pa, 5, size); // fill with junk
  if(pa == 0)
    panic("memory not enough");
  release(&pages_lock);
  // printf("%p, end:%p \n", (void*)pa, (void*)end);
  return (void*)pa;
}



//compute the free memory of kernel
uint64 computeFreeMemory(){
  struct run *p;
  acquire(&kmem.lock);
  p = kmem.freelist;
  uint64 size = 0;
  while(p != 0){
    p = p->next;
    size += PGSIZE;
  }
  release(&kmem.lock);
  return size;
}