#include "signal.h"

void saveContext(struct proc *p, struct u_context_t *user_context_ptr) {
    copyout(p->pagetable, (uint64)&user_context_ptr->ra, (char*)&p->trapframe->ra, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->sp, (char*)&p->trapframe->sp, 8);
    
    // Copying argument registers
    copyout(p->pagetable, (uint64)&user_context_ptr->a0, (char*)&p->trapframe->a0, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->a1, (char*)&p->trapframe->a1, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->a2, (char*)&p->trapframe->a2, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->a3, (char*)&p->trapframe->a3, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->a4, (char*)&p->trapframe->a4, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->a5, (char*)&p->trapframe->a5, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->a6, (char*)&p->trapframe->a6, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->a7, (char*)&p->trapframe->a7, 8);
    
    // Copying saved registers
    copyout(p->pagetable, (uint64)&user_context_ptr->s0, (char*)&p->trapframe->s0, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->s1, (char*)&p->trapframe->s1, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->s2, (char*)&p->trapframe->s2, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->s3, (char*)&p->trapframe->s3, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->s4, (char*)&p->trapframe->s4, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->s5, (char*)&p->trapframe->s5, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->s6, (char*)&p->trapframe->s6, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->s7, (char*)&p->trapframe->s7, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->s8, (char*)&p->trapframe->s8, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->s9, (char*)&p->trapframe->s9, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->s10, (char*)&p->trapframe->s10, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->s11, (char*)&p->trapframe->s11, 8);
    
    // Copying temporary registers
    copyout(p->pagetable, (uint64)&user_context_ptr->t0, (char*)&p->trapframe->t0, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->t1, (char*)&p->trapframe->t1, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->t2, (char*)&p->trapframe->t2, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->t3, (char*)&p->trapframe->t3, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->t4, (char*)&p->trapframe->t4, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->t5, (char*)&p->trapframe->t5, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->t6, (char*)&p->trapframe->t6, 8);
    copyout(p->pagetable, (uint64)&user_context_ptr->epc, (char*)&p->trapframe->epc, 8);
}

void restoreContext(struct proc *p, struct u_context_t *user_context_ptr) {
    // Copying return address and stack pointer
    copyin(p->pagetable, (char*)&p->trapframe->ra, (uint64)&user_context_ptr->ra, 8);
    copyin(p->pagetable, (char*)&p->trapframe->sp, (uint64)&user_context_ptr->sp, 8);

    // Copying saved registers
    copyin(p->pagetable, (char*)&p->trapframe->s0, (uint64)&user_context_ptr->s0, 8);
    copyin(p->pagetable, (char*)&p->trapframe->s1, (uint64)&user_context_ptr->s1, 8);
    copyin(p->pagetable, (char*)&p->trapframe->s2, (uint64)&user_context_ptr->s2, 8);
    copyin(p->pagetable, (char*)&p->trapframe->s3, (uint64)&user_context_ptr->s3, 8);
    copyin(p->pagetable, (char*)&p->trapframe->s4, (uint64)&user_context_ptr->s4, 8);
    copyin(p->pagetable, (char*)&p->trapframe->s5, (uint64)&user_context_ptr->s5, 8);
    copyin(p->pagetable, (char*)&p->trapframe->s6, (uint64)&user_context_ptr->s6, 8);
    copyin(p->pagetable, (char*)&p->trapframe->s7, (uint64)&user_context_ptr->s7, 8);
    copyin(p->pagetable, (char*)&p->trapframe->s8, (uint64)&user_context_ptr->s8, 8);
    copyin(p->pagetable, (char*)&p->trapframe->s9, (uint64)&user_context_ptr->s9, 8);
    copyin(p->pagetable, (char*)&p->trapframe->s10, (uint64)&user_context_ptr->s10, 8);
    copyin(p->pagetable, (char*)&p->trapframe->s11, (uint64)&user_context_ptr->s11, 8);
    
    // Copying temporary registers
    copyin(p->pagetable, (char*)&p->trapframe->t0, (uint64)&user_context_ptr->t0, 8);
    copyin(p->pagetable, (char*)&p->trapframe->t1, (uint64)&user_context_ptr->t1, 8);
    copyin(p->pagetable, (char*)&p->trapframe->t2, (uint64)&user_context_ptr->t2, 8);
    copyin(p->pagetable, (char*)&p->trapframe->t3, (uint64)&user_context_ptr->t3, 8);
    copyin(p->pagetable, (char*)&p->trapframe->t4, (uint64)&user_context_ptr->t4, 8);
    copyin(p->pagetable, (char*)&p->trapframe->t5, (uint64)&user_context_ptr->t5, 8);
    copyin(p->pagetable, (char*)&p->trapframe->t6, (uint64)&user_context_ptr->t6, 8);
    
    // Copying argument registers
    copyin(p->pagetable, (char*)&p->trapframe->a0, (uint64)&user_context_ptr->a0, 8);
    copyin(p->pagetable, (char*)&p->trapframe->a1, (uint64)&user_context_ptr->a1, 8);
    copyin(p->pagetable, (char*)&p->trapframe->a2, (uint64)&user_context_ptr->a2, 8);
    copyin(p->pagetable, (char*)&p->trapframe->a3, (uint64)&user_context_ptr->a3, 8);
    copyin(p->pagetable, (char*)&p->trapframe->a4, (uint64)&user_context_ptr->a4, 8);
    copyin(p->pagetable, (char*)&p->trapframe->a5, (uint64)&user_context_ptr->a5, 8);
    copyin(p->pagetable, (char*)&p->trapframe->a6, (uint64)&user_context_ptr->a6, 8);
    copyin(p->pagetable, (char*)&p->trapframe->a7, (uint64)&user_context_ptr->a7, 8);
    copyin(p->pagetable, (char*)&p->trapframe->epc, (uint64)&user_context_ptr->epc, 8);
}

void print_registers(struct proc *p) {
    printf("print the tramframe:\n");
    printf("ra: %p\n", (void*)p->trapframe->ra);
    printf("sp: %p\n", (void*)p->trapframe->sp);
    printf("epc: %p\n", (void*)p->trapframe->epc);

    printf("a0: %p\n", (void*)p->trapframe->a0);
    printf("a1: %p\n", (void*)p->trapframe->a1);
    printf("a2: %p\n", (void*)p->trapframe->a2);
    printf("a3: %p\n", (void*)p->trapframe->a3);
    printf("a4: %p\n", (void*)p->trapframe->a4);
    printf("a5: %p\n", (void*)p->trapframe->a5);
    printf("a6: %p\n", (void*)p->trapframe->a6);
    printf("a7: %p\n", (void*)p->trapframe->a7);
    
    printf("s0: %p\n", (void*)p->trapframe->s0);
    printf("s1: %p\n", (void*)p->trapframe->s1);
    printf("s2: %p\n", (void*)p->trapframe->s2);
    printf("s3: %p\n", (void*)p->trapframe->s3);
    printf("s4: %p\n", (void*)p->trapframe->s4);
    printf("s5: %p\n", (void*)p->trapframe->s5);
    printf("s6: %p\n", (void*)p->trapframe->s6);
    printf("s7: %p\n", (void*)p->trapframe->s7);
    printf("s8: %p\n", (void*)p->trapframe->s8);
    printf("s9: %p\n", (void*)p->trapframe->s9);
    printf("s10: %p\n", (void*)p->trapframe->s10);
    printf("s11: %p\n", (void*)p->trapframe->s11);
    
    printf("t0: %p\n", (void*)p->trapframe->t0);
    printf("t1: %p\n", (void*)p->trapframe->t1);
    printf("t2: %p\n", (void*)p->trapframe->t2);
    printf("t3: %p\n", (void*)p->trapframe->t3);
    printf("t4: %p\n", (void*)p->trapframe->t4);
    printf("t5: %p\n", (void*)p->trapframe->t5);
    printf("t6: %p\n", (void*)p->trapframe->t6);
    
    // Skipping tp and gp
}