#include "user/cothread.h"

// Switch context between two cothreads
void cothread_switch(uint64 old_c_ptr, uint64 new_c_ptr) {
    __asm__ volatile (
        // Save the registers of the current cothread (old_c_ptr)
        "sd s0, 0(%0)\n\t"
        "sd s1, 8(%0)\n\t"
        "sd s2, 16(%0)\n\t"
        "sd s3, 24(%0)\n\t"
        "sd s4, 32(%0)\n\t"
        "sd s5, 40(%0)\n\t"
        "sd s6, 48(%0)\n\t"
        "sd s7, 56(%0)\n\t"
        "sd s8, 64(%0)\n\t"
        "sd s9, 72(%0)\n\t"
        "sd s10, 80(%0)\n\t"
        "sd s11, 88(%0)\n\t"
        "sd ra, 96(%0)\n\t"
        "sd sp, 104(%0)\n\t"

        // Load the registers of the next cothread (new_c_ptr)
        "ld s0, 0(%1)\n\t"
        "ld s1, 8(%1)\n\t"
        "ld s2, 16(%1)\n\t"
        "ld s3, 24(%1)\n\t"
        "ld s4, 32(%1)\n\t"
        "ld s5, 40(%1)\n\t"
        "ld s6, 48(%1)\n\t"
        "ld s7, 56(%1)\n\t"
        "ld s8, 64(%1)\n\t"
        "ld s9, 72(%1)\n\t"
        "ld s10, 80(%1)\n\t"
        "ld s11, 88(%1)\n\t"
        "ld ra, 96(%1)\n\t"
        "ld sp, 104(%1)\n\t"
        : // No output operands
        : "r" (old_c_ptr), "r" (new_c_ptr) // Input operands
        : "memory" // Clobbered registers
    );
}

// Initialize the cothread system
void cothread_init(void) {
    current_cothread = &all_cothread[0];
    current_cothread->state = RUNNING;
}

// Schedule the next cothread to run
void cothread_schedule(void) {
    struct cothread *t, *next_cothread = 0;

    // Find another runnable cothread
    t = current_cothread + 1;
    for (int i = 0; i < MAX_cothread; i++) {
        if (t >= all_cothread + MAX_cothread)
            t = all_cothread;

        if (t->state == READY) {
            next_cothread = t;
            break;
        }
        t++;
    }

    if (next_cothread == 0) {
        printf("cothread_schedule: no runnable cothreads\n");
        exit(-1);
    }

    // Switch cothreads if necessary
    if (current_cothread != next_cothread) {
        next_cothread->state = RUNNING;
        struct cothread *old_cothread = current_cothread;
        current_cothread = next_cothread;
        cothread_switch((uint64)old_cothread, (uint64)next_cothread);
    } else {
        printf("no runnable cothread\n");
    }
}

// Create a new cothread
void cothread_create(void (*func)()) {
    struct cothread *t;
    int found = 0;

    for (t = all_cothread; t < all_cothread + MAX_cothread; t++) {
        if (t->state == FREE) {
            found = 1;
            break;
        }
    }

    if (!found) {
        printf("no available cothread!\n");
        while (1); // Halt if no cothread is available
    }

    t->state = READY;
    t->sp = (uint64)(t->stack) + STACK_SIZE;
    t->ra = (uint64)func;
}

// Yield the current cothread
void cothread_yield(void) {
    current_cothread->state = READY;
    cothread_schedule();
}