#include "kernel/include/types.h"
#include "kernel/include/stat.h"
#include "user/user.h"
#include "kernel/include/fcntl.h"
// #define SIMPLE_TEST
#ifdef SIMPLE_TEST


#define NUM_PROCESSES 10

void child_process(int id, int priority) {
    set_priority(priority); // 设置优先级
    
    for (volatile int j = 0; j < 100000000; j++)
        for(volatile int k = 0; k < 5; k++);

    // 打印进程信息和计数
    printf("Process %d (Priority %d)\n", id, priority);
    
    // 获取并打印运行时间和虚拟运行时间
    int runtime = get_runtime();
    int vruntime = get_vruntime();
    printf("get priority: %d\n", get_priority());
    printf("Process %d: Runtime = %d µs, VRuntime = %d µs\n", id, runtime, vruntime);
    
    // 模拟一些工作
        
    exit(0); // 结束子进程
}

int main(void) {
    for (int i = 0; i < NUM_PROCESSES; i++) {
        int pid = fork(); // 创建子进程
        if (pid < 0) {
            printf("Fork failed\n");
            exit(0);
        }
        if (pid == 0) {
            // 子进程
            child_process(i + 1, i); // 设置不同的优先级
        }
    }

    // 父进程等待所有子进程结束
    for (int i = 0; i < NUM_PROCESSES; i++) {
        wait(0);
    }

    printf("All child processes completed.\n");
    exit(0);
}

#endif

#ifndef SIMPLE_TEST

#define NUM_PROCESSES 50
#define GROUP_SIZE 10

void child_process(int id, int priority) {
    set_priority(priority); // 设置优先级
    // printf("child process %d created...", id);

    // 模拟一些工作
    for (volatile int j = 0; j < 100000000; j++)
        for (volatile int k = 0; k < 5; k++);

    // 获取并打印运行时间和虚拟运行时间
    int runtime = get_runtime();
    int vruntime = get_vruntime();
    
    // 输出结果格式为 priority, runtime, vruntime
    printf("%d, %d, %d\n", priority, runtime, vruntime);

    exit(0); // 结束子进程
}

int main(void) {
    printf("testing CFS scheduler....\n");
    for (int i = 0; i < NUM_PROCESSES; i++) {
        int priority = i / GROUP_SIZE; // 每10个进程共享一个优先级
        int pid = fork(); // 创建子进程
        if (pid == 0) {
            // 子进程
            child_process(i + 1, priority); // 设置优先级
        }
    }

    // 父进程等待所有子进程结束
    for (int i = 0; i < NUM_PROCESSES; i++) {
        wait(0);
    }

    printf("All child processes completed.\n");
    exit(0);
}

#endif