#include "userthread.h"


struct threadpool pool;

void printQueue(void);

void uthread_yield(void){
    sigalarm(0, 0);
    struct thread *t = pool.queue[pool.currentThreadId];
    if(t->state != FINISHED)t->state = READY;
    sigalarm(10, handler);

    uthread_scheduler();
}

void uthread_scheduler(void){
    sigalarm(0, 0);
    int i;
    int nextidx = pool.currentThreadId;
    if(pool.queue[pool.currentThreadId]->state == FINISHED){
        //FREE THAT THREAD AND MAINTAIN THE POOL
        if(nextidx != pool.thread_count - 1){
            if(nextidx != 0) free(pool.queue[nextidx]);
            pool.queue[nextidx] = pool.queue[pool.thread_count - 1];
            pool.thread_count--;
        }else{
            if(nextidx != 0) free(pool.queue[nextidx]);
            pool.thread_count--;
        }
    }
    
    //scheduling
    for(i = 0; i < pool.thread_count; ++i){
        nextidx = (i + pool.currentThreadId + 1) % pool.thread_count;
        if(pool.queue[nextidx]->state == READY){
            break;
        }
    }
    if(i != pool.thread_count){
        pool.currentThreadId = nextidx;
    }else{
        printf("no available thread!\n");
        printQueue();
        while(1);
    }

    sigalarm(10, handler);
}

void uthread_init() {
    memset(&pool, 0, sizeof(pool));
    pool.currentThreadId = 0;
    pool.thread_count = 1;
    pool.queue[0] = (struct thread*)malloc(sizeof(struct thread));
}


void handler(void* u_context_ptr){
    struct threadContext *tc = (struct threadContext*)(u_context_ptr);
    printf("thread[%d] context saved!\n", pool.currentThreadId);
    pool.queue[pool.currentThreadId]->context = *tc;
    // print_context(tc);
    uthread_yield();
    *tc = pool.queue[pool.currentThreadId]->context;
    printf("thread[%d] context restored!\n", pool.currentThreadId);
    sigreturn();
}

void threadwrapper(void (*func)(void *), void *arg){
    sigalarm(0, 0);
    if(pool.currentThreadId == 0){
        printQueue();
        while(1);
    }
    sigalarm(10, handler);
    func(arg);
    sigalarm(0, 0);
    pool.queue[pool.currentThreadId]->state = FINISHED;
    printQueue();
    sleep(200);
                                    /*WE CANNOT FREE THIS THREAD HERE SINCE WE STILL NEED ITS STACK*/
    // uthread_scheduler();            /*VERY IMPORTANT!
                                    //   this thread will never return, since it will be DISCARDED*/
}


//return tid
int uthread_create(void (*func)(void *), void *arg) {
    sigalarm(0, 0);
    if (pool.thread_count >= MAXTHREADNUM) {
        printf("uthread warning! no available thread.\n");
        return -1;
    }
    int tid = pool.thread_count++;
    struct thread *p;
    pool.queue[tid] = p = (struct thread*)malloc(PGSIZE);
    p->id = tid;
    printf("create thread[%d] successful!\n", tid);
    p->state = READY;
    p->context.ra = (uint64)threadwrapper;
    p->context.sp = ((uint64)(p)) + PGSIZE;
    p->context.a0 = (uint64) func;
    p->context.a1 = (uint64) arg;
    sigalarm(10, handler);
    return tid;
}

uint32 uthread_id(void){
    sigalarm(0, 0);
    uint32 id = pool.queue[pool.currentThreadId]->id;
    sigalarm(10, handler);
    return id;
}

void uthread_run(void){
    pool.queue[0]->state = RUNNING;
    sigalarm(10, handler);
    while(1){
        for(int i = 0; i < 10000000; i++){
            if(i == 0) printf("main thread\n");
        }
    };
}


void print_context(struct threadContext *p) {
    printf("print the context\n");
    printf("ra: %p\n", (void*)p->ra);
    printf("sp: %p\n", (void*)p->sp);
    printf("epc: %p\n", (void*)p->epc);

    printf("a0: %p\n", (void*)p->a0);
    printf("a1: %p\n", (void*)p->a1);
    printf("a2: %p\n", (void*)p->a2);
    printf("a3: %p\n", (void*)p->a3);
    printf("a4: %p\n", (void*)p->a4);
    printf("a5: %p\n", (void*)p->a5);
    printf("a6: %p\n", (void*)p->a6);
    printf("a7: %p\n", (void*)p->a7);
    
    printf("s0: %p\n", (void*)p->s0);
    printf("s1: %p\n", (void*)p->s1);
    printf("s2: %p\n", (void*)p->s2);
    printf("s3: %p\n", (void*)p->s3);
    printf("s4: %p\n", (void*)p->s4);
    printf("s5: %p\n", (void*)p->s5);
    printf("s6: %p\n", (void*)p->s6);
    printf("s7: %p\n", (void*)p->s7);
    printf("s8: %p\n", (void*)p->s8);
    printf("s9: %p\n", (void*)p->s9);
    printf("s10: %p\n", (void*)p->s10);
    printf("s11: %p\n", (void*)p->s11);
    
    printf("t0: %p\n", (void*)p->t0);
    printf("t1: %p\n", (void*)p->t1);
    printf("t2: %p\n", (void*)p->t2);
    printf("t3: %p\n", (void*)p->t3);
    printf("t4: %p\n", (void*)p->t4);
    printf("t5: %p\n", (void*)p->t5);
    printf("t6: %p\n", (void*)p->t6);
    
    // Skipping tp and gp
}

void printQueue(void){
    for(int i = 0; i < pool.thread_count; ++i){
        switch(pool.queue[i]->state){
            case FINISHED: printf("Thread[%d] FINISHED\n", pool.queue[i]->id); break;
            case READY: printf("Thread[%d] READY\n", pool.queue[i]->id); break;
            case RUNNING: printf("Thread[%d] RUNNING\n", pool.queue[i]->id); break;
        }
    }
}